@@ -7,7 +7,9 @@ import logging
 import grpc
 
 from preseem import PreseemGrpcClient
-from preseem_protobuf.network_poller.network_poller_pb2 import NetworkElementUpdate, SendUpdateBatchRequest, GetUpdatesRequest
+from preseem_protobuf.network_poller.network_poller_pb2 import (
+    NetworkElementUpdate, SendUpdateBatchRequest, GetUpdatesRequest,
+    NetworkElementDiscovery, SendDiscoveryBatchRequest, GetDiscoveryRequest)
 from preseem_protobuf.network_poller.network_poller_pb2_grpc import NetworkPollerServiceStub
 
 class NetworkPollerGrpcClient:
@@ -24,20 +26,30 @@ class NetworkPollerGrpcClient:
             self._client = None  # for testing
         self.host = host
         self._svc = None
-        self._msg_q = None
-        self._task = None
+        self._msg_q_neu = None #asyncio.Queue(5000)
+        self._task_neu = None
         self._filters = []
-        self.drop_count = 0
+        self.drop_count_neu = 0
+        self._msg_q_ned = None
+        self._task_ned = None
+        self.drop_count_ned = 0
 
     async def close(self):
         """Close this object and cancel the client task."""
-        if self._task:
-            self._task.cancel()
+        if self._task_neu:
+            self._task_neu.cancel()
             try:
-                await self._task
+                await self._task_neu
             except asyncio.CancelledError:
                 pass
-            self._task = None
+            self._task_neu = None
+        if self._task_ned:
+            self._task_ned.cancel()
+            try:
+                await self._task_ned
+            except asyncio.CancelledError:
+                pass
+            self._task_ned = None
         if self._client:
             self._client = None
 
@@ -52,7 +64,18 @@ class NetworkPollerGrpcClient:
             req.company_uuid = company_uuid
         return await self._client._grpc_op(self._svc.GetUpdates, req, collect_results=True)
 
-    async def _post(self, messages):
+    async def get_discovery(self, company_uuid=None):
+        if not self._client:
+            return
+        if not self._svc:
+            await self._client.connect()
+            self._svc = NetworkPollerServiceStub(self._client._channel)
+        req = GetDiscoveryRequest()
+        if company_uuid:
+            req.company_uuid = company_uuid
+        return await self._client._grpc_op(self._svc.GetDiscovery, req, collect_results=True)
+
+    async def _post_neu(self, messages):
         """Post a list of NetworkElementUpdate messages"""
         if not self._client:
             return
@@ -63,7 +86,18 @@ class NetworkPollerGrpcClient:
         req.data.extend(messages)
         return await self._client._grpc_op(self._svc.SendUpdateBatch, req, timeout=self.POST_TIMEOUT)
 
-    async def _service_queue(self):
+    async def _post_ned(self, messages):
+        """Post a list of NetworkElementDiscovery messages"""
+        if not self._client:
+            return
+        if not self._svc:
+            await self._client.connect()
+            self._svc = NetworkPollerServiceStub(self._client._channel)
+        req = SendDiscoveryBatchRequest()
+        req.data.extend(messages)
+        return await self._client._grpc_op(self._svc.SendDiscoveryBatch, req, timeout=self.POST_TIMEOUT)
+
+    async def _service_queue(self, msg_q, post_fn, type):
         """Service the message queue"""
         n = 0
         try:
@@ -73,19 +107,24 @@ class NetworkPollerGrpcClient:
             while True:
                 if messages:
                     try:
-                        await self._post(messages)
+                        await post_fn(messages)
                         messages = []
                         size = 0
                     except asyncio.CancelledError:
                         raise
                     except grpc.RpcError as err:
-                        logging.warning('Error [%s] posting NetworkElementUpdate to %s: %s.  Queue size: %s.', err.code(), self.host, err.details(), self._msg_q.qsize())
+                        if type is NetworkElementUpdate:
+                            logging.warning('Error [%s] posting NetworkElementUpdate to %s: %s.  Queue size: %s.',
+                                                            err.code(), self.host, err.details(), self._msg_q_neu.qsize())
+                        elif type is NetworkElementDiscovery:
+                            logging.warning('Error [%s] posting NetworkElementDiscovery to %s: %s.  Queue size: %s.',
+                                                            err.code(), self.host, err.details(), self._msg_q_ned.qsize())
                         await asyncio.sleep(self.RETRY_HOLDOFF_TIME)
                         continue
                     except Exception as err:
                         # if it fails, we'll retry until it succeeds.  So we're
                         # assuming the message is ok or we will never proceed.
-                        logging.error('Error posting NetworkElementUpdate: %s', err)
+                        logging.error('Error posting %s: %s', type.__name__, err)
                         await asyncio.sleep(self.RETRY_HOLDOFF_TIME)
                         continue
                 try:
@@ -94,10 +133,10 @@ class NetworkPollerGrpcClient:
                     while True:
                         if messages:
                             # build the batch until the holdoff time
-                            msg = await asyncio.wait_for(self._msg_q.get(), t - loop.time())
+                            msg = await asyncio.wait_for(msg_q.get(), t - loop.time())
                         else:
-                            msg = await self._msg_q.get()
-                        self._msg_q.task_done()
+                            msg = await msg_q.get()
+                        msg_q.task_done()
                         size += msg.ByteSize()
                         messages.append(msg)
                         if size > self.MAX_MESSAGE_SIZE:
@@ -108,7 +147,7 @@ class NetworkPollerGrpcClient:
                     # try to send any in-progress message
                     try:
                         if messages:
-                            await self._post(messages)
+                            await post_fn(messages)
                     finally:
                         raise
         except asyncio.CancelledError:
@@ -119,26 +158,44 @@ class NetworkPollerGrpcClient:
         self._filters.append(f)
 
     async def post(self, msg, id_key=None):
-        """Push a NetworkElementUpdate message.  Optionally pass in an id_key
+        """Push a NetworkElementUpdate / NetworkElementDiscovery message.  Optionally pass in an id_key
            which will be passed along to any message filters."""
-        assert isinstance(msg, NetworkElementUpdate)
-        if not self._task:
-            self._msg_q = asyncio.Queue(5000)
-            self._task = asyncio.get_event_loop().create_task(self._service_queue())
-        try:
-            # Make a copy to make sure the caller can't change it after this
+        assert isinstance(msg, (NetworkElementUpdate, NetworkElementDiscovery))
+        type = None
+        if isinstance(msg, NetworkElementUpdate):
+            type = NetworkElementUpdate
             msg_copy = NetworkElementUpdate()
-            msg_copy.CopyFrom(msg)
-            for f in self._filters:
-                msg_copy = f(msg_copy, id_key)
-                if msg_copy is None:  # filter out this message
-                    return
-            self._msg_q.put_nowait(msg_copy)
+        elif isinstance(msg, NetworkElementDiscovery):
+            type = NetworkElementDiscovery
+            msg_copy = NetworkElementDiscovery()
+        if not self._task_neu:
+            self._msg_q_neu = asyncio.Queue(5000)
+            self._task_neu = asyncio.get_event_loop().create_task(self._service_queue(self._msg_q_neu, self._post_neu, type))
+        if not self._task_ned:
+            self._msg_q_ned = asyncio.Queue(5000)
+            self._task_ned = asyncio.get_event_loop().create_task(self._service_queue(self._msg_q_ned, self._post_ned, type))
+        # Make a copy to make sure the caller can't change it after this
+        msg_copy.CopyFrom(msg)
+        try:
+            if type is NetworkElementUpdate:
+                for f in self._filters:     # Apply filter only on NetworkElementUpdate
+                    msg_copy = f(msg_copy, id_key)
+                    if msg_copy is None:  # filter out this message
+                        return
+                self._msg_q_neu.put_nowait(msg_copy)
+            elif type is NetworkElementDiscovery:
+                self._msg_q_ned.put_nowait(msg_copy)
         except asyncio.QueueFull:  # send queue is full, drop the request
-            self.drop_count += 1
-            if str(self.drop_count)[0] != str(self.drop_count -1)[0]:
-                # log when we're dropping, but don't spam the log
-                logging.warning('NetworkElementUpdate queue full, %s dropped', self.drop_count)
+            if type is NetworkElementUpdate:
+                self.drop_count_neu += 1
+                if str(self.drop_count_neu)[0] != str(self.drop_count_neu -1)[0]:
+                    # log when we're dropping, but don't spam the log
+                    logging.warning('NetworkElementUpdate queue full, %s dropped', self.drop_count_neu)
+            elif type is NetworkElementDiscovery:
+                self.drop_count_ned += 1
+                if str(self.drop_count_ned)[0] != str(self.drop_count_ned -1)[0]:
+                    # log when we're dropping, but don't spam the log
+                    logging.warning('NetworkElementDiscovery queue full, %s dropped', self.drop_count_ned)
 
 
 class FakeNetworkPollerGrpcClient(NetworkPollerGrpcClient):
@@ -151,12 +208,18 @@ class FakeNetworkPollerGrpcClient(NetworkPollerGrpcClient):
         self.verbose = verbose
         self.fut = None
 
-    async def _post(self, messages):
+    async def _post_neu(self, messages):
         """Post a list of NetworkElementUpdate messages"""
         # TODO pretty-print messages if verbose > 0
         self.fut.set_result(messages)
         self.fut = None
 
+    async def _post_ned(self, messages):
+        """Post a list of NetworkElementDiscovery messages"""
+        # TODO pretty-print messages if verbose > 0
+        self.fut.set_result(messages)
+        self.fut = None
+
     async def post(self, msg, id_key=None):
         await super().post(msg, id_key)
         fut = self.fut or asyncio.get_event_loop().create_future()
