@@ -117,7 +117,7 @@ def _pb_request(op, reqtype, **kwargs):
 async def _op(model, op, reqtype, delete=False, **kwargs):
     """Wrapper function to perform a GRPC operation on the model. (asyncio)"""
     req = _pb_request(op, reqtype, **kwargs)
-    company_uuid = kwargs.get('company_uuid') or kwargs.get('uuid') if reqtype.__module__ == 'company_pb2' else None
+    company_uuid = kwargs.get('company_uuid') or (kwargs.get('uuid') if reqtype.__module__ == 'company_pb2' else None)
     md = (('company-uuid', str(UUID(bytes=company_uuid))),) if company_uuid else None
     r = await model._client._grpc_op(op, req, collect_results=True, metadata=md)
     del_uuid = kwargs.get('uuid') if delete else None
